/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btcore.h,v 1.11 2002/09/20 14:42:15 kds Exp $

   HCI access library

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/

#ifndef	_BTCORE_H
#define	_BTCORE_H

#include <fcntl.h>
#include <pthread.h>
#include <net/if.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <syslog.h>

__BEGIN_DECLS

#include <affix/hci_types.h>
#include <affix/bluetooth.h>

extern int argc;
extern char **argv;
extern int btlog_log;
extern char *affix_version;
extern unsigned long btlog_mask;
extern int btlog_options;

extern char	btdev[IFNAMSIZ];
#define LINK_BLUETOOTH	0
#define LINK_TCP	1
#define LINK_IRDA	2
extern int	linkmode;
extern int	sdpmode;

extern unsigned char	hci_errno;
extern char 		*hci_errlist[];
extern char 		*lmp_compid[];

struct affix_tupla {
	unsigned int	value;
	char		*match;
	char		*str;
};


extern struct affix_tupla debug_flags_map[];

extern struct affix_tupla codServiceClassMnemonic[];
extern struct affix_tupla codMajorClassMnemonic[];
extern struct affix_tupla codMinorComputerMnemonic[];
extern struct affix_tupla codMinorPhoneMnemonic[];
extern struct affix_tupla codMinorAudioMnemonic[];

extern struct affix_tupla pkt_type_map[];
extern struct affix_tupla sec_level_map[];
extern struct affix_tupla sec_mode_map[];


#define NUM_LMPCOMPID	37



#define BTDEBUG_MODULE_CONTROL	0x01
#define BTDEBUG_MODULE_SDP	0x02
#define BTDEBUG_MODULE_OBEX	0x04
#define BTDEBUG_MODULE_HFP	0x08

#ifndef BTDEBUG_MODULE 
#define BTDEBUG_MODULE 0
#endif

#ifdef CONFIG_AFFIX_DEBUG
#define _BTDEBUG(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, fmt "\n", ##args);\
				} while (0)

#define _BTDEBUGn(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, fmt, ##args);\
				} while (0)

#define BTDEBUGn(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, __FUNCTION__": " fmt, ##args);\
				} while (0)
#define BTDEBUG(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_printf(LOG_DEBUG, __FUNCTION__": " fmt "\n", ##args);\
				} while (0)

#define BTDUMP(data, len)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_hexdump(LOG_DEBUG, __FUNCTION__, data, len);\
				} while (0)
#else
#define _BTDEBUG(args...)
#define BTDEBUG(args...)
#define BTDUMP(data, len)
#endif

#define BTERROR(fmt, args...)		btlog_printf(LOG_ERR, __FUNCTION__ ": " fmt "\n", ##args)
#define BTWARN(fmt, args...)		btlog_printf(LOG_WARNING, __FUNCTION__ ": " fmt "\n", ##args)
#define BTNOTICE(fmt, args...)		btlog_printf(LOG_NOTICE, __FUNCTION__ ": " fmt "\n", ##args)
#define BTINFO(fmt, args...)		btlog_printf(LOG_INFO, __FUNCTION__ ": " fmt "\n", ##args)


// General purpose functions

int str2bda(BD_ADDR *p, char *str);
void bda2str(char *str, BD_ADDR *p);
void btlog_printf(int pr, const char *fmt, ...);
void btlog_hexdump(int priority, const char *fname, const unsigned char *data, int len);
int daemonize(void);

static inline int btsys_socket(int domain, int type, int protocol)
{
	return socket(domain, type, protocol);
}

static inline int btsys_ioctl(int fd, int cmd, void *arg)
{
	return ioctl(fd, cmd, arg);
}

static inline int btsys_close(int fd)
{
	return close(fd);
}

static inline int hci_close(int fd)
{
	return btsys_close(fd);
}

static inline int hci_exec_cmd(int fd, void *cmd, __u64 mask, int flags, void *event)
{
	struct hci_cmd_arg	cp;
	int			err;

	cp.cmd = cmd;
	cp.flags = flags;
	cp.event_mask = mask;
	cp.event = event;

	err = btsys_ioctl(fd, BTIOC_EXEC, &cp);
	if (err)
		return err;
	return cp.status;
}

static inline int hci_listen_event(int fd, __u64 mask)
{
	return btsys_ioctl(fd, BTIOC_LISTEN, &mask);
}

static inline int hci_recv_event(int fd, void *event, int *hd)
{
	struct hci_event_arg	ep;
	int			err;
	
	ep.event = event;
	
	err = btsys_ioctl(fd, BTIOC_RECV_EVENT, &ep);
	if (err)
		return err;
	if (hd)
		*hd = ep.hd;
	return 0;
}

static inline int hci_recv_msg(int fd, __u8 *param)
{
	return btsys_ioctl(fd, BTIOC_RECV_MSG, param);
}

static inline char *hci_errormsg(int errn)
{
	return hci_errlist[errn];
}

static inline unsigned char hci_last_error(void)
{
	unsigned char	ret = hci_errno;
	hci_errno = 0;
	return ret;
}


/* general hci */

static inline int hci_set_mode(int fd, int mode)
{
	struct affix_ctl_attr	ctl;
	ctl.mode = mode;
	return btsys_ioctl(fd, BTIOC_SET_CTL, &ctl);
}

static inline int hci_get_attr(int fd, struct hci_dev_attr *attr)
{
	return btsys_ioctl(fd, BTIOC_GET_HCI, attr);
}

static inline int hci_set_attr(int fd, struct hci_dev_attr *attr)
{
	return btsys_ioctl(fd, BTIOC_SET_HCI, attr);
}

static inline int hci_get_version(int fd, struct affix_version *ver)
{
	return btsys_ioctl(fd, BTIOC_GET_VERSION, ver);
}


/* Control */
int hci_get_devs(int *devs);
int hci_add_pin(BD_ADDR *bda, int Length, __u8 *Code);
int hci_remove_pin(BD_ADDR *bda);
int hci_remove_key(BD_ADDR *bda);

int hci_set_dbmask(__u32 dbmask);
int hci_get_dbmask(__u32 *dbmask);

int hci_open_uart(char *name, int speed, int flags, int proto);
int hci_close_uart(char *name);

int hci_set_audio(int fd, int mode, __u16 setting);
int hci_get_conn(int fd, BD_ADDR *bda);



/* socket stuff */

static inline int affix_getmtu(int fd)
{
	int		err, mtu;
	socklen_t	len;

	err = getsockopt(fd, SOL_AFFIX, BTSO_MTU, &mtu, &len);
	if (err < 0)
		return err;

	return mtu;
}

/* L2CAP */
static inline int l2cap_getmtu(int fd)
{
	return affix_getmtu(fd);
}

static inline int l2cap_setmtu(int fd, int mtu)
{
	return setsockopt(fd, SOL_AFFIX, BTSO_MTU, &mtu, sizeof(mtu));
}

static inline int l2cap_flush(int fd)
{
	return ioctl(fd, SIOCL2CAP_FLUSH, 0);
}

int l2cap_sendping(int fd, char *data, int size, int type);
int l2cap_ping(int fd, char *data, int size);

static inline int l2cap_singleping(int fd, char *data, int size)
{
	return l2cap_sendping(fd, data, size, L2CAP_SINGLEPING);
}


/* RFCOMM stuff */
int rfcomm_disconnect(int line);
int rfcomm_get_ports(struct rfcomm_port *pi, int size);

static inline int rfcomm_bind_port(int fd, int line)
{
	int	err;

	err = ioctl(fd, SIOCRFCOMM_BINDPORT, &line);
	if (err < 0)
		return err;

	return line;
}

static inline int rfcomm_set_type(int fd, int type)
{
	return ioctl(fd, SIOCRFCOMM_SETTYPE, &type);
}


int affix_init_pan(char *name, int mode);

__END_DECLS

#endif
