/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: obex.h,v 1.1 2002/07/12 08:27:12 kds Exp $

   OBEX client lib 

   Fixes:
   		Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/		

#ifndef OBEX_CLIENT_H
#define OBEX_CLIENT_H

#include <termios.h>
#include <glib.h>
#include <openobex/obex.h>

#include <affix/utils.h>

#if defined(CONFIG_AFFIX_SDP)
#include "sdp.h"
#include "sdpclt.h"
#endif

#define OBEX_LS		1
#define OBEX_PUT	2
#define OBEX_GET	3
#define OBEX_SETPATH	4
#define OBEX_PUSH	5
#define OBEX_REMOVE	6

/* Number of bytes passed at one time to OBEX */
#define OBEX_STREAM_CHUNK	4096

typedef struct {
	obex_t		*handle;
	obex_ctrans_t	custfunc;	/* store callbacks */
        const char	*portname;
        int		rfd, wfd;
        struct termios	oldtio, newtio;
	/* receiver state */
        char		inputbuf[OBEX_STREAM_CHUNK];
} obex_link_t;

typedef struct {
	obex_link_t	link;
	obex_t		*handle;
	int		fd;		/* rfcomm socket */
        gboolean	serverdone;
       	gboolean	clientdone;
        gchar		*get_name;	/* Name of last get-request */

	guint32		con_id;		/* connection ide */
	int		rsp;		/* error code */
	int		opcode;		/* internal operation code */
	char		*buf;		/* response storage place */
	int		sfd;		/* stream fd */
} obexclt_t;

/* Bluetooth OBEX link */
obex_t *btobex_init(obex_link_t *link, obex_event_t eventcb, guint flags);
void btobex_cleanup(obex_link_t *link, gboolean force);
int btobex_setport(obex_link_t *gt, const gchar *port);
int btobex_setsocket(obex_link_t *gt, const int fd);


char *obex_error(int rsp);
int obex_push(void *addr, char *name);
int obex_get(obexclt_t *handle, char *name);
int obex_put(obexclt_t *clt, char *name);
int obex_remove(obexclt_t *clt, char *name);
int obex_browse(obexclt_t *clt, char *name, char *buf);

#define SETPATH_CREATE	0x0001
int __obex_setpath(obexclt_t *clt, char *path, int flags);

static inline int obex_setpath(obexclt_t *clt, char *path)
{
	return __obex_setpath(clt, path, 0);
}

static inline int obex_mkdir(obexclt_t *clt, char *path)
{
	int	err;
	err = __obex_setpath(clt, path, SETPATH_CREATE);
	if (!err)
		err = __obex_setpath(clt, "..", 0);
	return err;
}


obexclt_t *obex_connect(void *addr, int *err);
int obex_disconnect(obexclt_t *clt);

#endif	/*	BTCTL_H */

